"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BotCommandError = exports.BotCommandHandler = exports.botCommand = exports.botCommandSymbol = void 0;
require("reflect-metadata");
const markdown_it_1 = __importDefault(require("markdown-it"));
const string_argv_1 = __importDefault(require("string-argv"));
const md = new markdown_it_1.default();
exports.botCommandSymbol = Symbol("botCommandMetadata");
function botCommand(options) {
    Reflect.metadata(exports.botCommandSymbol, options);
}
exports.botCommand = botCommand;
class BotCommandHandler {
    constructor(prototype, instance, prefix) {
        this.instance = instance;
        this.prefix = prefix;
        let content = "Commands:\n";
        const botCommands = {};
        Object.getOwnPropertyNames(prototype).forEach(propetyKey => {
            var _a;
            const b = Reflect.getMetadata(exports.botCommandSymbol, prototype, propetyKey);
            if (!b) {
                // Not a bot command function.
                return;
            }
            const requiredArgs = b.requiredArgs.join(" ");
            const optionalArgs = ((_a = b.optionalArgs) === null || _a === void 0 ? void 0 : _a.map((arg) => `[${arg}]`).join(" ")) || "";
            content += ` - \`££PREFIX££${b.name}\` ${requiredArgs} ${optionalArgs} - ${b.help}\n`;
            // We know that this is safe.
            botCommands[b.name] = {
                fn: prototype[propetyKey],
                requiredArgs: b.requiredArgs,
                optionalArgs: b.optionalArgs,
            };
        });
        this.helpMarkdown = content;
        this.botCommands = botCommands;
    }
    getHelpMessage(cmdPrefix) {
        return {
            msgtype: "m.notice",
            body: this.helpMarkdown,
            formatted_body: md.render(this.helpMarkdown).replace(/££PREFIX££/g, cmdPrefix || ""),
            format: "org.matrix.custom.html"
        };
    }
    /**
     * @param userId The userId who sent the bot command.
     * @param command The command string, including the prefix
     * @param botCommands The set of bot commands in use.
     * @param obj
     * @param prefix
     * @returns True if the command was handled by this handler instance, otherwise false.
     */
    async handleCommand(userCommand, userId) {
        // The processor may require a prefix (like `!github `). Check for it
        // and strip away if found.
        if (this.prefix) {
            if (!userCommand.startsWith(this.prefix)) {
                return false;
            }
            userCommand = userCommand.substring(this.prefix.length);
        }
        const parts = string_argv_1.default(userCommand);
        // This loop is a little complex:
        // We want to find the most likely candiate for handling this command
        // which we do so by joining together the whole command string and
        // matching against any commands with the same name.
        // If we can't find any, we strip away the last arg and try again.
        // E.g. In the case of `add one + two`, we would search for:
        // - `add one + two`
        // - `add one +`
        // - `add one`
        // - `add`
        // We iterate backwards so that command trees can be respected.
        for (let i = parts.length; i > 0; i--) {
            const cmdPrefix = parts.slice(0, i).join(" ").toLowerCase();
            // We have a match!
            const command = this.botCommands[cmdPrefix];
            if (command) {
                if (command.requiredArgs.length > parts.length - i) {
                    throw new BotCommandError("Missing arguments", "Missing required arguments for this command");
                }
                await command.fn.apply(this.instance, [{
                        userId,
                        args: parts.slice(i),
                    }]);
                return true;
            }
        }
        return false;
    }
}
exports.BotCommandHandler = BotCommandHandler;
class BotCommandError extends Error {
    constructor(error, humanText) {
        super(typeof error === "string" ? error : error.message);
        this.humanText = humanText;
        if (typeof error !== "string") {
            this.stack = error.stack;
        }
    }
}
exports.BotCommandError = BotCommandError;
//# sourceMappingURL=bot-commands.js.map